﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;
using System.Xml;
using Assets.Utils;
using UnityEngine;

public class XMLParser
{
    private XmlDocument XML;
    public XMLParser()
    {
        var files = Directory.GetFiles(Application.dataPath + "/data");
        foreach (var xmlFile in files.Where(f => f.EndsWith(".xml")))
        {
            // new xdoc instance 
            XML = new XmlDocument();

            //load up the xml from the location 
            XML.Load(xmlFile);
        }
    }
    public List<EventSession> ParseFile()
    {
        var sessions = XML.DocumentElement.ChildNodes;
        var eventSessions = new List<EventSession>();

        // nodes inside xml
        foreach (XmlNode node in sessions)
        {
            switch (node.Name)
            {
                case "massageBank":
                    ParseMassageBank(node, null, true);
                    break;
                case "eventSession":
                    var eventSession = ParseEventSession(node);
                    eventSessions.Add(eventSession);
                    break;
            }
        }
        return eventSessions;
    }

    private EventSession ParseEventSession(XmlNode node)
    {
        var eventSession = new EventSession();
        var eventName = node.Attributes.ValueOrNull("name");
        eventSession.EventName = eventName;
        var eventLength = int.Parse(node.Attributes.ValueOrNull("length"));
        eventSession.Length = eventLength;
        var firstMassage = GetFirstMassage(node);
        eventSession.StartingMassage = firstMassage;
        foreach (XmlNode xmlNode in node)
        {
            switch (xmlNode.Name)
            {
                case "leader":
                    var leaderTuple = ParseLeader(xmlNode);
                    if (leaderTuple.Left == 1)
                    {
                        eventSession.Leader1 = leaderTuple.Right;
                    }
                    else
                    {
                        eventSession.Leader2 = leaderTuple.Right;
                    }
                    break;
                case "massageBank":
                    var massageBank = ParseMassageBank(xmlNode, eventSession);
                    eventSession.Leader1MassageBank = massageBank.Left;
                    eventSession.Leader2MassageBank = massageBank.Right;
                    break;
            }
        }
        return eventSession;
    }

    private Tuple<MassageBank, MassageBank> ParseMassageBank(XmlNode xmlNode, EventSession evnt, bool isGlobal = false)
    {

        var bank = new MassageBank();
        Leader leader1 = null;
        Leader leader2 = null;
        foreach (XmlNode node in xmlNode)
        {
            switch (node.Name)
            {
                case "leader":
                    var leaderTuple = ParseLeader(node);
                    if (leaderTuple.Left == 1)
                    {
                        leader1 = leaderTuple.Right;
                    }
                    else
                    {
                        leader2 = leaderTuple.Right;
                    }
                    break;
                case "massage":
                    var massage = ParseMassage(node);
                    bank.Massages.Add(massage);
                    break;
            }
        }
        if (evnt != null)
        {
            leader2 = evnt.Leader2;
            leader1 = evnt.Leader1;
        }
        var bank1 = new MassageBank();
        bank1.Massages.AddRange(bank.Massages.Where(massage => massage.senderLeader == leader1));
        var bank2 = new MassageBank();
        bank2.Massages.AddRange(bank.Massages.Where(massage => massage.senderLeader == leader2));
        if (isGlobal)
        {
            MassageBank.AddMassageBank(leader1, leader2, bank1);
            MassageBank.AddMassageBank(leader2, leader1, bank2);

        }
        return new Tuple<MassageBank, MassageBank>(bank1, bank2);
    }

    private Tuple<int, Leader> ParseLeader(XmlNode xmlNode)
    {

        var leaderName = xmlNode.Attributes.ValueOrNull("load");//rename field to "name"?
        var leader = GameObject.FindObjectsOfType<Leader>().First(x =>
        {
            Debug.Log(leaderName);
            Debug.Log(x.LeaderName);
            return x.LeaderName == leaderName;
        });
        var number = int.Parse(xmlNode.Attributes.ValueOrNull("num"));
        return new Tuple<int, Leader>(number, leader);
    }

    private Massage GetFirstMassage(XmlNode evnt)
    {
        foreach (XmlNode node in evnt)
        {
            if (node.Name == "massage" &&
                node.Attributes.isValue("start", "true"))
            {
                return ParseMassage(node);
            }
        }
        return null;
    }

    private Massage ParseMassage(XmlNode node)
    {
        var text = node.Attributes.ValueOrNull("text");
        var leaderName = node.Attributes.ValueOrNull("leader");
        var leader = GameObject.FindObjectsOfType<Leader>().First(x => x.LeaderName == leaderName);
        var options = new List<Option>();
        foreach (XmlNode response in node)
        {
            //It's assumed that nodes contains only responses
            var anger = int.Parse(response.Attributes.ValueOrNull("Anger"));
            var word = "";
            var feedback = "";
            foreach (XmlNode xmlNode in response)
            {
                switch (xmlNode.Name)
                {
                    case "text":
                        word = xmlNode.InnerText;
                        break;
                    case "feedback":
                        feedback = xmlNode.InnerText;
                        break;
                }
            }
            options.Add(new Option(word, anger, feedback));
        }
        var massage = new ChangeAbleMassage(0, text, options);
        massage.senderLeader = leader;
        return massage;
    }
}
